# CheapEmote release 2 by David Bowland
# ./addons/eventscripts/cheapemote/cheapemote.py

# >>> To configure this addon please see cheapemote.cfg (created whent he addon is first loaded) <<<

"""
Provides chat triggers to display emote icons above player heads. Requires only ES 2.0+
"""


import cfglib
import es
import gamethread
import os.path
import re
import time

import psyco
psyco.full()


###


info = es.AddonInfo()
info.name          = 'CheapEmote'
info.version       = '2'
info.basename      = 'cheapemote'
info.url           = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author        = 'SuperDave'
info.specialthanks = 'Einlanzers, Mattie, Venjax, and Wonder'


###


config = cfglib.AddonCFG(es.getAddonPath(info.basename) + '/' + info.basename + '.cfg')

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename':info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('')
config.text('Thanks to Einlanzers, Mattie, Venjax, and Wonder for their work on the original script!')
config.text('')
config.text('All models included with CheapEmote are by Venjax. The original Emote can be found here:')
config.text('http://addons.eventscripts.com/addons/view/emote')
config.text('\n')

config.text('***** Delay option *****')
icon_delay = config.cvar('cheapemote_expiredelay', 5, 'Number of seconds to show emote icon')
config.text('\n')

config.text('***** Sound options *****')
sound_delay  = config.cvar('cheapemote_sound_delay',       5, 'Number of seconds after triggering a sound a player must wait to trigger another sound')
sound_volume = config.cvar('cheapemote_sound_volume',      1, 'Volume to play sounds (0.0 = no sound, 1.0 = full volume)')
attenuation  = config.cvar('cheapemote_sound_attenuation', 0, 'Attenuation to use when playing sounds (0.0 = entire map, 1.0 = single player)')
config.text('\n')

config.text('***** Icon/trigger options *****')
config.text('')
config.text('Use the following server command to add emote icons/triggers:')
config.text('')
config.text('cheapemote_addicon <"trigger"> <"icon path"> ["sound relative to ./sound/"]')
config.text('')
config.text('Each icon requires:')
config.text('   ./materials/models/extras/<icon path>/speech_info.vmt')
config.text('   ./materials/models/extras/<icon path>/speech_info.vtf')
config.text('   ./models/extras/<icon path>/info_speech.dx80.vtx')
config.text('   ./models/extras/<icon path>/info_speech.dx90.vtx')
config.text('   ./models/extras/<icon path>/info_speech.mdl')
config.text('   ./models/extras/<icon path>/info_speech.phy')
config.text('   ./models/extras/<icon path>/info_speech.sw.vtx')
config.text('   ./models/extras/<icon path>/info_speech.vvd')
config.text('\n')

config.text('Place your cheapemote_addicon commands below:')
config.command('cheapemote_addicon')
config.text('\n')

config.text('Uncomment the following default options if you wish to use them:')
config.text('')
config.text('cheapemote_addicon "m!" "mbang"')
config.text('cheapemote_addicon ":)" "smile"')
config.text('cheapemote_addicon ":(" "frown"')
config.text('cheapemote_addicon "lol" "grin"')
config.text('cheapemote_addicon ":o" "ooh"')
config.text('cheapemote_addicon ":P" "ras"')
config.text('\n')

config.text('The following server command can be used to remove emote triggers:')
config.text('')
config.text('cheapemote_removeicon <"trigger">')
config.text('')
config.text('Due to the fact emote icons/triggers can be added and removed above, this command will not be used by most users.')

config.write()


es_lastgive = es.ServerVar('eventscripts_lastgive') # Provided by ES for the last created entity


###


def getEyeAngles():
   """Returns the eye angle property based on game"""
   eye_angles = {'cstrike':'CCSPlayer.m_angEyeAngles[%s]', 'dod':'CDODPlayer.m_angEyeAngles[%s]', 'hl2mp':'CHL2MP_Player.m_angEyeAngles[%s]'}
   gamename   = str(es.ServerVar('eventscripts_gamedir')).replace('\\', '/').split('/')[~0]

   es.dbgmsg(1, 'CheapEmote: Current game - \"%s\"' % gamename)

   return eye_angles.get(gamename, eye_angles['cstrike'])

eyeangle_prop = getEyeAngles()


###


class IconManager(object):
   icons     = {}
   soundpath = es.ServerVar('eventscripts_gamedir') + '/sound/'

   def __init__(self):
      self.icons = {}

   def addIcon(self, trigger, icon, sound=''):
      """Adds the icon and trigger to the dictionary of icons"""
      self.icons[trigger] = {'icon':icon, 'regex':re.compile('([ \\.!\\@\\$%\\^\\&\\*\\(\\),\";\\?-]|^)' + re.escape(trigger) + '([ \\.!\\@\\$%\\^\\&\\*\\(\\),\";\\?-]|$)'), 'sound':sound}

   def removeIcon(self, trigger):
      """Removes the trigger from the dictionary of icons"""
      if trigger in self.icons:
         del self.icons[trigger]


   def findIcon(self, userid, text):
      """Displays any icons corresponding with the text"""
      if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
         for trigger in self.icons:
            current_icon = self.icons[trigger]
            if current_icon['regex'].search(text):
               getPlayer(userid).createIcon(current_icon['icon'], current_icon['sound'])
               break


   def markDownloads(self):
      """Marks icon materials for download"""
      for trigger in self.icons:
         current_icon = self.icons[trigger]

         for path in ('materials/models/extras/%s/speech_info.vmt', 'materials/models/extras/%s/speech_info.vtf', 'models/extras/%s/info_speech.dx80.vtx', 'models/extras/%s/info_speech.dx90.vtx', 'models/extras/%s/info_speech.mdl', 'models/extras/%s/info_speech.phy', 'models/extras/%s/info_speech.sw.vtx', 'models/extras/%s/info_speech.vvd'):
            es.stringtable('downloadables', path % current_icon['icon'])

         if os.path.isfile(self.soundpath + current_icon['sound']) if current_icon['sound'] else False:
            es.stringtable('downloadables', 'sound/' + current_icon['sound'])

icons = IconManager() # Public instance of IconManager


class EmoteIcon(object):
   userid    = 0
   index     = 0
   lastsound = 0

   def __init__(self, userid):
      self.userid = userid


   def createIcon(self, icon, sound):
      """Creates the specified icon above the player"""
      self.removeIcon()

      now = time.time()
      if sound and now - self.lastsound >= sound_delay:
         es.emitsound('player', self.userid, sound, sound_volume, attenuation)
         self.lastsound = now

      eye_angles = tuple(es.getplayerprop(self.userid, eyeangle_prop % x) for x in (0, 1))
      es.server.queuecmd('es_xsetang %s 90 0 0' % self.userid)

      es.server.queuecmd('es_xprop_dynamic_create %(userid)s \"extras/%(icon)s/info_speech.mdl\";cheapemote_setindex %(userid)s;es_xentsetname %(userid)s emote_bubble_%(userid)s' % {'userid':self.userid, 'icon':icon})
      location = es.getplayerlocation(self.userid)
      es.server.queuecmd('es_xfire %s emote_bubble_%s addoutput \"origin %s %s %s\"' % (self.userid, self.userid, location[0], location[1], location[2] + 96))
      es.server.queuecmd('es_xfire %(userid)s emote_bubble_%(userid)s SetParent !activator;es_xfire %(userid)s emote_bubble_%(userid)s SetAnimation idle;es_xfire %(userid)s emote_bubble_%(userid)s addoutput \"angles 0 0 0\";es_xfire %(userid)s emote_bubble_%(userid)s addoutput \"solid 0\"' % {'userid':self.userid})

      es.server.queuecmd('es_xsetang %s %s %s 0' % ((self.userid,) + eye_angles))

      gamethread.delayedname(icon_delay, 'emote_%s' % self.userid, self.removeIcon, False)

   def removeIcon(self, remove_delay=True):
      """Removes the player's icon"""
      if not self.index: return

      entlist = es.createentitylist(self.index)
      if (entlist[self.index]['classname'] == 'prop_dynamic') if entlist else False:
         es.server.queuecmd('es_xremove %s' % self.index)

      if remove_delay:
         self.removeDelay()
      self.index = 0

   def removeDelay(self):
      """Removes any outstanding icon delay"""
      if self.index:
         gamethread.cancelDelayed('emote_%s' % self.userid)

   def setIndex(self):
      self.index = int(es_lastgive)


def setindex_cmd():
   userid = int(es.getargv(1))
   if userid in players:
      getPlayer(userid).setIndex()

if not es.exists('command', 'cheapemote_setindex'):
   es.regcmd('cheapemote_setindex', 'cheapemote/setindex_cmd', 'CheapEmote: INTERNAL USE ONLY!')


###


players = {} # EmoteIcon instances


def getPlayer(userid):
   """Returns the EmoteIcon instance corresponding to the userid"""
   userid = int(userid)
   if userid not in players:
      players[userid] = EmoteIcon(userid)

   return players[userid]


def removePlayer(userid):
   """Removes the EmoteIcon instance corresponding to the userid"""
   userid = int(userid)
   if userid in players:
      players[userid].removeIcon()
      del players[userid]


def clearAllIcons(remove=False):
   """Clears all icons"""
   for userid in players:
      if remove:
         players[userid].removeIcon()

      else:
         players[userid].removeDelay()

   players.clear()


###


def load():
   config.execute()

   if str(es.ServerVar('eventscripts_currentmap')):
      icons.markDownloads()


def es_map_start(event_var):
   clearAllIcons()

   icons.markDownloads()


def round_start(event_var):
   clearAllIcons()


def dod_round_start(event_var):
   clearAllIcons()


def player_say(event_var):
   icons.findIcon(int(event_var['userid']), event_var['text'])


def player_death(event_var):
   removePlayer(event_var['userid'])


def player_disconnect(event_var):
   removePlayer(event_var['userid'])


def unload():
   """Clears CheapEmote cvars"""
   clearAllIcons(True)

   for cvar in config.getCvars():
      es.ServerVar(cvar).set(0)


###


def addicon_cmd():
   """
   cheapemote_addicon <"trigger"> <"icon path"> ["sound relative to ./sound/"]
   Server command wrapper for adding emote icons and triggers
   """
   arg_count = es.getargc()
   if arg_count in (3, 4):
      icons.addIcon(es.getargv(1), es.getargv(2), es.getargv(3) if arg_count == 4 else '')

   else:
      es.dbgmsg(0, 'Syntax: cheapemote_addicon <\"trigger\"> <\"icon path\"> [\"sound relative to ./sound/\"]')

if not es.exists('command', 'cheapemote_addicon'):
   es.regcmd('cheapemote_addicon', 'cheapemote/addicon_cmd', 'cheapemote_addicon <\"trigger\"> <\"icon path\">  [\"sound relative to ./sound/\"]\nAdds an emote icon')


def removeicon_cmd():
   """
   cheapemote_removeicon <"trigger">
   Server command wrapper for removing emote triggers
   """
   if es.getargc() == 2:
      icons.removeIcon(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: cheapemote_removeicon <\"trigger\">')

if not es.exists('command', 'cheapemote_removeicon'):
   es.regcmd('cheapemote_removeicon', 'cheapemote/removeicon_cmd', 'cheapemote_removeicon <\"trigger\">\nRemoves an emote icon')